# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins
import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities

__all__ = [
    'GetCartResult',
    'AwaitableGetCartResult',
    'get_cart',
    'get_cart_output',
]

@pulumi.output_type
class GetCartResult:
    """
    A collection of values returned by getCart.
    """
    def __init__(__self__, assign=None, cart_id=None, description=None, expire=None, id=None, items=None, ovh_subsidiary=None, read_only=None):
        if assign and not isinstance(assign, bool):
            raise TypeError("Expected argument 'assign' to be a bool")
        pulumi.set(__self__, "assign", assign)
        if cart_id and not isinstance(cart_id, str):
            raise TypeError("Expected argument 'cart_id' to be a str")
        pulumi.set(__self__, "cart_id", cart_id)
        if description and not isinstance(description, str):
            raise TypeError("Expected argument 'description' to be a str")
        pulumi.set(__self__, "description", description)
        if expire and not isinstance(expire, str):
            raise TypeError("Expected argument 'expire' to be a str")
        pulumi.set(__self__, "expire", expire)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if items and not isinstance(items, list):
            raise TypeError("Expected argument 'items' to be a list")
        pulumi.set(__self__, "items", items)
        if ovh_subsidiary and not isinstance(ovh_subsidiary, str):
            raise TypeError("Expected argument 'ovh_subsidiary' to be a str")
        pulumi.set(__self__, "ovh_subsidiary", ovh_subsidiary)
        if read_only and not isinstance(read_only, bool):
            raise TypeError("Expected argument 'read_only' to be a bool")
        pulumi.set(__self__, "read_only", read_only)

    @property
    @pulumi.getter
    def assign(self) -> Optional[builtins.bool]:
        return pulumi.get(self, "assign")

    @property
    @pulumi.getter(name="cartId")
    def cart_id(self) -> builtins.str:
        """
        Cart identifier
        """
        return pulumi.get(self, "cart_id")

    @property
    @pulumi.getter
    def description(self) -> Optional[builtins.str]:
        return pulumi.get(self, "description")

    @property
    @pulumi.getter
    def expire(self) -> builtins.str:
        return pulumi.get(self, "expire")

    @property
    @pulumi.getter
    def id(self) -> builtins.str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def items(self) -> Sequence[builtins.int]:
        """
        Items of your cart
        """
        return pulumi.get(self, "items")

    @property
    @pulumi.getter(name="ovhSubsidiary")
    def ovh_subsidiary(self) -> builtins.str:
        return pulumi.get(self, "ovh_subsidiary")

    @property
    @pulumi.getter(name="readOnly")
    def read_only(self) -> builtins.bool:
        """
        Indicates if the cart has already been validated
        """
        return pulumi.get(self, "read_only")


class AwaitableGetCartResult(GetCartResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetCartResult(
            assign=self.assign,
            cart_id=self.cart_id,
            description=self.description,
            expire=self.expire,
            id=self.id,
            items=self.items,
            ovh_subsidiary=self.ovh_subsidiary,
            read_only=self.read_only)


def get_cart(assign: Optional[builtins.bool] = None,
             description: Optional[builtins.str] = None,
             expire: Optional[builtins.str] = None,
             ovh_subsidiary: Optional[builtins.str] = None,
             opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetCartResult:
    """
    Use this data source to create a temporary order cart to retrieve information order cart products.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_ovh as ovh

    my_account = ovh.Me.get_me()
    my_cart = ovh.Order.get_cart(ovh_subsidiary=my_account.ovh_subsidiary)
    ```


    :param builtins.bool assign: Assign a shopping cart to a logged in client. Values can be `true` or `false`.
    :param builtins.str description: Description of your cart
    :param builtins.str expire: Expiration time (format: 2006-01-02T15:04:05+00:00)
    :param builtins.str ovh_subsidiary: OVHcloud Subsidiary. Country of OVHcloud legal entity you'll be billed by. List of supported subsidiaries available on API at [/1.0/me.json under `models.nichandle.OvhSubsidiaryEnum`](https://eu.api.ovh.com/1.0/me.json)
    """
    __args__ = dict()
    __args__['assign'] = assign
    __args__['description'] = description
    __args__['expire'] = expire
    __args__['ovhSubsidiary'] = ovh_subsidiary
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('ovh:Order/getCart:getCart', __args__, opts=opts, typ=GetCartResult).value

    return AwaitableGetCartResult(
        assign=pulumi.get(__ret__, 'assign'),
        cart_id=pulumi.get(__ret__, 'cart_id'),
        description=pulumi.get(__ret__, 'description'),
        expire=pulumi.get(__ret__, 'expire'),
        id=pulumi.get(__ret__, 'id'),
        items=pulumi.get(__ret__, 'items'),
        ovh_subsidiary=pulumi.get(__ret__, 'ovh_subsidiary'),
        read_only=pulumi.get(__ret__, 'read_only'))
def get_cart_output(assign: Optional[pulumi.Input[Optional[builtins.bool]]] = None,
                    description: Optional[pulumi.Input[Optional[builtins.str]]] = None,
                    expire: Optional[pulumi.Input[Optional[builtins.str]]] = None,
                    ovh_subsidiary: Optional[pulumi.Input[builtins.str]] = None,
                    opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetCartResult]:
    """
    Use this data source to create a temporary order cart to retrieve information order cart products.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_ovh as ovh

    my_account = ovh.Me.get_me()
    my_cart = ovh.Order.get_cart(ovh_subsidiary=my_account.ovh_subsidiary)
    ```


    :param builtins.bool assign: Assign a shopping cart to a logged in client. Values can be `true` or `false`.
    :param builtins.str description: Description of your cart
    :param builtins.str expire: Expiration time (format: 2006-01-02T15:04:05+00:00)
    :param builtins.str ovh_subsidiary: OVHcloud Subsidiary. Country of OVHcloud legal entity you'll be billed by. List of supported subsidiaries available on API at [/1.0/me.json under `models.nichandle.OvhSubsidiaryEnum`](https://eu.api.ovh.com/1.0/me.json)
    """
    __args__ = dict()
    __args__['assign'] = assign
    __args__['description'] = description
    __args__['expire'] = expire
    __args__['ovhSubsidiary'] = ovh_subsidiary
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('ovh:Order/getCart:getCart', __args__, opts=opts, typ=GetCartResult)
    return __ret__.apply(lambda __response__: GetCartResult(
        assign=pulumi.get(__response__, 'assign'),
        cart_id=pulumi.get(__response__, 'cart_id'),
        description=pulumi.get(__response__, 'description'),
        expire=pulumi.get(__response__, 'expire'),
        id=pulumi.get(__response__, 'id'),
        items=pulumi.get(__response__, 'items'),
        ovh_subsidiary=pulumi.get(__response__, 'ovh_subsidiary'),
        read_only=pulumi.get(__response__, 'read_only')))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins
import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = ['StorageArgs', 'Storage']

@pulumi.input_type
class StorageArgs:
    def __init__(__self__, *,
                 region_name: pulumi.Input[builtins.str],
                 service_name: pulumi.Input[builtins.str],
                 encryption: Optional[pulumi.Input['StorageEncryptionArgs']] = None,
                 limit: Optional[pulumi.Input[builtins.float]] = None,
                 marker: Optional[pulumi.Input[builtins.str]] = None,
                 name: Optional[pulumi.Input[builtins.str]] = None,
                 owner_id: Optional[pulumi.Input[builtins.float]] = None,
                 prefix: Optional[pulumi.Input[builtins.str]] = None,
                 replication: Optional[pulumi.Input['StorageReplicationArgs']] = None,
                 versioning: Optional[pulumi.Input['StorageVersioningArgs']] = None):
        """
        The set of arguments for constructing a Storage resource.
        :param pulumi.Input[builtins.str] region_name: Region name
        :param pulumi.Input[builtins.str] service_name: Service name
        :param pulumi.Input['StorageEncryptionArgs'] encryption: Encryption configuration
        :param pulumi.Input[builtins.float] limit: Limit the number of objects returned (1000 maximum, defaults to 1000)
        :param pulumi.Input[builtins.str] marker: Key to start with when listing objects
        :param pulumi.Input[builtins.str] name: Container name
        :param pulumi.Input[builtins.float] owner_id: Container owner user ID
        :param pulumi.Input[builtins.str] prefix: List objects whose key begins with this prefix
        :param pulumi.Input['StorageReplicationArgs'] replication: Replication configuration
        :param pulumi.Input['StorageVersioningArgs'] versioning: Versioning configuration
        """
        pulumi.set(__self__, "region_name", region_name)
        pulumi.set(__self__, "service_name", service_name)
        if encryption is not None:
            pulumi.set(__self__, "encryption", encryption)
        if limit is not None:
            pulumi.set(__self__, "limit", limit)
        if marker is not None:
            pulumi.set(__self__, "marker", marker)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if owner_id is not None:
            pulumi.set(__self__, "owner_id", owner_id)
        if prefix is not None:
            pulumi.set(__self__, "prefix", prefix)
        if replication is not None:
            pulumi.set(__self__, "replication", replication)
        if versioning is not None:
            pulumi.set(__self__, "versioning", versioning)

    @property
    @pulumi.getter(name="regionName")
    def region_name(self) -> pulumi.Input[builtins.str]:
        """
        Region name
        """
        return pulumi.get(self, "region_name")

    @region_name.setter
    def region_name(self, value: pulumi.Input[builtins.str]):
        pulumi.set(self, "region_name", value)

    @property
    @pulumi.getter(name="serviceName")
    def service_name(self) -> pulumi.Input[builtins.str]:
        """
        Service name
        """
        return pulumi.get(self, "service_name")

    @service_name.setter
    def service_name(self, value: pulumi.Input[builtins.str]):
        pulumi.set(self, "service_name", value)

    @property
    @pulumi.getter
    def encryption(self) -> Optional[pulumi.Input['StorageEncryptionArgs']]:
        """
        Encryption configuration
        """
        return pulumi.get(self, "encryption")

    @encryption.setter
    def encryption(self, value: Optional[pulumi.Input['StorageEncryptionArgs']]):
        pulumi.set(self, "encryption", value)

    @property
    @pulumi.getter
    def limit(self) -> Optional[pulumi.Input[builtins.float]]:
        """
        Limit the number of objects returned (1000 maximum, defaults to 1000)
        """
        return pulumi.get(self, "limit")

    @limit.setter
    def limit(self, value: Optional[pulumi.Input[builtins.float]]):
        pulumi.set(self, "limit", value)

    @property
    @pulumi.getter
    def marker(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Key to start with when listing objects
        """
        return pulumi.get(self, "marker")

    @marker.setter
    def marker(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "marker", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Container name
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter(name="ownerId")
    def owner_id(self) -> Optional[pulumi.Input[builtins.float]]:
        """
        Container owner user ID
        """
        return pulumi.get(self, "owner_id")

    @owner_id.setter
    def owner_id(self, value: Optional[pulumi.Input[builtins.float]]):
        pulumi.set(self, "owner_id", value)

    @property
    @pulumi.getter
    def prefix(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        List objects whose key begins with this prefix
        """
        return pulumi.get(self, "prefix")

    @prefix.setter
    def prefix(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "prefix", value)

    @property
    @pulumi.getter
    def replication(self) -> Optional[pulumi.Input['StorageReplicationArgs']]:
        """
        Replication configuration
        """
        return pulumi.get(self, "replication")

    @replication.setter
    def replication(self, value: Optional[pulumi.Input['StorageReplicationArgs']]):
        pulumi.set(self, "replication", value)

    @property
    @pulumi.getter
    def versioning(self) -> Optional[pulumi.Input['StorageVersioningArgs']]:
        """
        Versioning configuration
        """
        return pulumi.get(self, "versioning")

    @versioning.setter
    def versioning(self, value: Optional[pulumi.Input['StorageVersioningArgs']]):
        pulumi.set(self, "versioning", value)


@pulumi.input_type
class _StorageState:
    def __init__(__self__, *,
                 created_at: Optional[pulumi.Input[builtins.str]] = None,
                 encryption: Optional[pulumi.Input['StorageEncryptionArgs']] = None,
                 limit: Optional[pulumi.Input[builtins.float]] = None,
                 marker: Optional[pulumi.Input[builtins.str]] = None,
                 name: Optional[pulumi.Input[builtins.str]] = None,
                 objects: Optional[pulumi.Input[Sequence[pulumi.Input['StorageObjectArgs']]]] = None,
                 objects_count: Optional[pulumi.Input[builtins.float]] = None,
                 objects_size: Optional[pulumi.Input[builtins.float]] = None,
                 owner_id: Optional[pulumi.Input[builtins.float]] = None,
                 prefix: Optional[pulumi.Input[builtins.str]] = None,
                 region: Optional[pulumi.Input[builtins.str]] = None,
                 region_name: Optional[pulumi.Input[builtins.str]] = None,
                 replication: Optional[pulumi.Input['StorageReplicationArgs']] = None,
                 service_name: Optional[pulumi.Input[builtins.str]] = None,
                 versioning: Optional[pulumi.Input['StorageVersioningArgs']] = None,
                 virtual_host: Optional[pulumi.Input[builtins.str]] = None):
        """
        Input properties used for looking up and filtering Storage resources.
        :param pulumi.Input[builtins.str] created_at: The date and timestamp when the resource was created
        :param pulumi.Input['StorageEncryptionArgs'] encryption: Encryption configuration
        :param pulumi.Input[builtins.float] limit: Limit the number of objects returned (1000 maximum, defaults to 1000)
        :param pulumi.Input[builtins.str] marker: Key to start with when listing objects
        :param pulumi.Input[builtins.str] name: Container name
        :param pulumi.Input[Sequence[pulumi.Input['StorageObjectArgs']]] objects: Container objects
        :param pulumi.Input[builtins.float] objects_count: Container total objects count
        :param pulumi.Input[builtins.float] objects_size: Container total objects size (bytes)
        :param pulumi.Input[builtins.float] owner_id: Container owner user ID
        :param pulumi.Input[builtins.str] prefix: List objects whose key begins with this prefix
        :param pulumi.Input[builtins.str] region: Container region
        :param pulumi.Input[builtins.str] region_name: Region name
        :param pulumi.Input['StorageReplicationArgs'] replication: Replication configuration
        :param pulumi.Input[builtins.str] service_name: Service name
        :param pulumi.Input['StorageVersioningArgs'] versioning: Versioning configuration
        :param pulumi.Input[builtins.str] virtual_host: Container virtual host
        """
        if created_at is not None:
            pulumi.set(__self__, "created_at", created_at)
        if encryption is not None:
            pulumi.set(__self__, "encryption", encryption)
        if limit is not None:
            pulumi.set(__self__, "limit", limit)
        if marker is not None:
            pulumi.set(__self__, "marker", marker)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if objects is not None:
            pulumi.set(__self__, "objects", objects)
        if objects_count is not None:
            pulumi.set(__self__, "objects_count", objects_count)
        if objects_size is not None:
            pulumi.set(__self__, "objects_size", objects_size)
        if owner_id is not None:
            pulumi.set(__self__, "owner_id", owner_id)
        if prefix is not None:
            pulumi.set(__self__, "prefix", prefix)
        if region is not None:
            pulumi.set(__self__, "region", region)
        if region_name is not None:
            pulumi.set(__self__, "region_name", region_name)
        if replication is not None:
            pulumi.set(__self__, "replication", replication)
        if service_name is not None:
            pulumi.set(__self__, "service_name", service_name)
        if versioning is not None:
            pulumi.set(__self__, "versioning", versioning)
        if virtual_host is not None:
            pulumi.set(__self__, "virtual_host", virtual_host)

    @property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        The date and timestamp when the resource was created
        """
        return pulumi.get(self, "created_at")

    @created_at.setter
    def created_at(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "created_at", value)

    @property
    @pulumi.getter
    def encryption(self) -> Optional[pulumi.Input['StorageEncryptionArgs']]:
        """
        Encryption configuration
        """
        return pulumi.get(self, "encryption")

    @encryption.setter
    def encryption(self, value: Optional[pulumi.Input['StorageEncryptionArgs']]):
        pulumi.set(self, "encryption", value)

    @property
    @pulumi.getter
    def limit(self) -> Optional[pulumi.Input[builtins.float]]:
        """
        Limit the number of objects returned (1000 maximum, defaults to 1000)
        """
        return pulumi.get(self, "limit")

    @limit.setter
    def limit(self, value: Optional[pulumi.Input[builtins.float]]):
        pulumi.set(self, "limit", value)

    @property
    @pulumi.getter
    def marker(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Key to start with when listing objects
        """
        return pulumi.get(self, "marker")

    @marker.setter
    def marker(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "marker", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Container name
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def objects(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['StorageObjectArgs']]]]:
        """
        Container objects
        """
        return pulumi.get(self, "objects")

    @objects.setter
    def objects(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['StorageObjectArgs']]]]):
        pulumi.set(self, "objects", value)

    @property
    @pulumi.getter(name="objectsCount")
    def objects_count(self) -> Optional[pulumi.Input[builtins.float]]:
        """
        Container total objects count
        """
        return pulumi.get(self, "objects_count")

    @objects_count.setter
    def objects_count(self, value: Optional[pulumi.Input[builtins.float]]):
        pulumi.set(self, "objects_count", value)

    @property
    @pulumi.getter(name="objectsSize")
    def objects_size(self) -> Optional[pulumi.Input[builtins.float]]:
        """
        Container total objects size (bytes)
        """
        return pulumi.get(self, "objects_size")

    @objects_size.setter
    def objects_size(self, value: Optional[pulumi.Input[builtins.float]]):
        pulumi.set(self, "objects_size", value)

    @property
    @pulumi.getter(name="ownerId")
    def owner_id(self) -> Optional[pulumi.Input[builtins.float]]:
        """
        Container owner user ID
        """
        return pulumi.get(self, "owner_id")

    @owner_id.setter
    def owner_id(self, value: Optional[pulumi.Input[builtins.float]]):
        pulumi.set(self, "owner_id", value)

    @property
    @pulumi.getter
    def prefix(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        List objects whose key begins with this prefix
        """
        return pulumi.get(self, "prefix")

    @prefix.setter
    def prefix(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "prefix", value)

    @property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Container region
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "region", value)

    @property
    @pulumi.getter(name="regionName")
    def region_name(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Region name
        """
        return pulumi.get(self, "region_name")

    @region_name.setter
    def region_name(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "region_name", value)

    @property
    @pulumi.getter
    def replication(self) -> Optional[pulumi.Input['StorageReplicationArgs']]:
        """
        Replication configuration
        """
        return pulumi.get(self, "replication")

    @replication.setter
    def replication(self, value: Optional[pulumi.Input['StorageReplicationArgs']]):
        pulumi.set(self, "replication", value)

    @property
    @pulumi.getter(name="serviceName")
    def service_name(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Service name
        """
        return pulumi.get(self, "service_name")

    @service_name.setter
    def service_name(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "service_name", value)

    @property
    @pulumi.getter
    def versioning(self) -> Optional[pulumi.Input['StorageVersioningArgs']]:
        """
        Versioning configuration
        """
        return pulumi.get(self, "versioning")

    @versioning.setter
    def versioning(self, value: Optional[pulumi.Input['StorageVersioningArgs']]):
        pulumi.set(self, "versioning", value)

    @property
    @pulumi.getter(name="virtualHost")
    def virtual_host(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Container virtual host
        """
        return pulumi.get(self, "virtual_host")

    @virtual_host.setter
    def virtual_host(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "virtual_host", value)


class Storage(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 encryption: Optional[pulumi.Input[Union['StorageEncryptionArgs', 'StorageEncryptionArgsDict']]] = None,
                 limit: Optional[pulumi.Input[builtins.float]] = None,
                 marker: Optional[pulumi.Input[builtins.str]] = None,
                 name: Optional[pulumi.Input[builtins.str]] = None,
                 owner_id: Optional[pulumi.Input[builtins.float]] = None,
                 prefix: Optional[pulumi.Input[builtins.str]] = None,
                 region_name: Optional[pulumi.Input[builtins.str]] = None,
                 replication: Optional[pulumi.Input[Union['StorageReplicationArgs', 'StorageReplicationArgsDict']]] = None,
                 service_name: Optional[pulumi.Input[builtins.str]] = None,
                 versioning: Optional[pulumi.Input[Union['StorageVersioningArgs', 'StorageVersioningArgsDict']]] = None,
                 __props__=None):
        """
        ## Example Usage

        ```python
        import pulumi
        import pulumi_ovh as ovh

        storage = ovh.cloud_project.Storage("storage",
            service_name="<public cloud project ID>",
            region_name="GRA",
            name="my-storage",
            versioning={
                "status": "enabled",
            })
        ```

        ## Import

        A storage in a public cloud project can be imported using the `service_name`, `region_name` and `name` attributes. Using the following configuration:

        terraform

        import {

          id = "<service_name>/<region_name>/<name>"

          to = ovh_cloud_project_storage.storage

        }

        You can then run:

        bash

        $ pulumi preview -generate-config-out=storage.tf

        $ pulumi up

        The file `storage.tf` will then contain the imported resource's configuration, that can be copied next to the `import` block above. See https://developer.hashicorp.com/terraform/language/import/generating-configuration for more details.

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Union['StorageEncryptionArgs', 'StorageEncryptionArgsDict']] encryption: Encryption configuration
        :param pulumi.Input[builtins.float] limit: Limit the number of objects returned (1000 maximum, defaults to 1000)
        :param pulumi.Input[builtins.str] marker: Key to start with when listing objects
        :param pulumi.Input[builtins.str] name: Container name
        :param pulumi.Input[builtins.float] owner_id: Container owner user ID
        :param pulumi.Input[builtins.str] prefix: List objects whose key begins with this prefix
        :param pulumi.Input[builtins.str] region_name: Region name
        :param pulumi.Input[Union['StorageReplicationArgs', 'StorageReplicationArgsDict']] replication: Replication configuration
        :param pulumi.Input[builtins.str] service_name: Service name
        :param pulumi.Input[Union['StorageVersioningArgs', 'StorageVersioningArgsDict']] versioning: Versioning configuration
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: StorageArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        ## Example Usage

        ```python
        import pulumi
        import pulumi_ovh as ovh

        storage = ovh.cloud_project.Storage("storage",
            service_name="<public cloud project ID>",
            region_name="GRA",
            name="my-storage",
            versioning={
                "status": "enabled",
            })
        ```

        ## Import

        A storage in a public cloud project can be imported using the `service_name`, `region_name` and `name` attributes. Using the following configuration:

        terraform

        import {

          id = "<service_name>/<region_name>/<name>"

          to = ovh_cloud_project_storage.storage

        }

        You can then run:

        bash

        $ pulumi preview -generate-config-out=storage.tf

        $ pulumi up

        The file `storage.tf` will then contain the imported resource's configuration, that can be copied next to the `import` block above. See https://developer.hashicorp.com/terraform/language/import/generating-configuration for more details.

        :param str resource_name: The name of the resource.
        :param StorageArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(StorageArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 encryption: Optional[pulumi.Input[Union['StorageEncryptionArgs', 'StorageEncryptionArgsDict']]] = None,
                 limit: Optional[pulumi.Input[builtins.float]] = None,
                 marker: Optional[pulumi.Input[builtins.str]] = None,
                 name: Optional[pulumi.Input[builtins.str]] = None,
                 owner_id: Optional[pulumi.Input[builtins.float]] = None,
                 prefix: Optional[pulumi.Input[builtins.str]] = None,
                 region_name: Optional[pulumi.Input[builtins.str]] = None,
                 replication: Optional[pulumi.Input[Union['StorageReplicationArgs', 'StorageReplicationArgsDict']]] = None,
                 service_name: Optional[pulumi.Input[builtins.str]] = None,
                 versioning: Optional[pulumi.Input[Union['StorageVersioningArgs', 'StorageVersioningArgsDict']]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = StorageArgs.__new__(StorageArgs)

            __props__.__dict__["encryption"] = encryption
            __props__.__dict__["limit"] = limit
            __props__.__dict__["marker"] = marker
            __props__.__dict__["name"] = name
            __props__.__dict__["owner_id"] = owner_id
            __props__.__dict__["prefix"] = prefix
            if region_name is None and not opts.urn:
                raise TypeError("Missing required property 'region_name'")
            __props__.__dict__["region_name"] = region_name
            __props__.__dict__["replication"] = replication
            if service_name is None and not opts.urn:
                raise TypeError("Missing required property 'service_name'")
            __props__.__dict__["service_name"] = service_name
            __props__.__dict__["versioning"] = versioning
            __props__.__dict__["created_at"] = None
            __props__.__dict__["objects"] = None
            __props__.__dict__["objects_count"] = None
            __props__.__dict__["objects_size"] = None
            __props__.__dict__["region"] = None
            __props__.__dict__["virtual_host"] = None
        super(Storage, __self__).__init__(
            'ovh:CloudProject/storage:Storage',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            created_at: Optional[pulumi.Input[builtins.str]] = None,
            encryption: Optional[pulumi.Input[Union['StorageEncryptionArgs', 'StorageEncryptionArgsDict']]] = None,
            limit: Optional[pulumi.Input[builtins.float]] = None,
            marker: Optional[pulumi.Input[builtins.str]] = None,
            name: Optional[pulumi.Input[builtins.str]] = None,
            objects: Optional[pulumi.Input[Sequence[pulumi.Input[Union['StorageObjectArgs', 'StorageObjectArgsDict']]]]] = None,
            objects_count: Optional[pulumi.Input[builtins.float]] = None,
            objects_size: Optional[pulumi.Input[builtins.float]] = None,
            owner_id: Optional[pulumi.Input[builtins.float]] = None,
            prefix: Optional[pulumi.Input[builtins.str]] = None,
            region: Optional[pulumi.Input[builtins.str]] = None,
            region_name: Optional[pulumi.Input[builtins.str]] = None,
            replication: Optional[pulumi.Input[Union['StorageReplicationArgs', 'StorageReplicationArgsDict']]] = None,
            service_name: Optional[pulumi.Input[builtins.str]] = None,
            versioning: Optional[pulumi.Input[Union['StorageVersioningArgs', 'StorageVersioningArgsDict']]] = None,
            virtual_host: Optional[pulumi.Input[builtins.str]] = None) -> 'Storage':
        """
        Get an existing Storage resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[builtins.str] created_at: The date and timestamp when the resource was created
        :param pulumi.Input[Union['StorageEncryptionArgs', 'StorageEncryptionArgsDict']] encryption: Encryption configuration
        :param pulumi.Input[builtins.float] limit: Limit the number of objects returned (1000 maximum, defaults to 1000)
        :param pulumi.Input[builtins.str] marker: Key to start with when listing objects
        :param pulumi.Input[builtins.str] name: Container name
        :param pulumi.Input[Sequence[pulumi.Input[Union['StorageObjectArgs', 'StorageObjectArgsDict']]]] objects: Container objects
        :param pulumi.Input[builtins.float] objects_count: Container total objects count
        :param pulumi.Input[builtins.float] objects_size: Container total objects size (bytes)
        :param pulumi.Input[builtins.float] owner_id: Container owner user ID
        :param pulumi.Input[builtins.str] prefix: List objects whose key begins with this prefix
        :param pulumi.Input[builtins.str] region: Container region
        :param pulumi.Input[builtins.str] region_name: Region name
        :param pulumi.Input[Union['StorageReplicationArgs', 'StorageReplicationArgsDict']] replication: Replication configuration
        :param pulumi.Input[builtins.str] service_name: Service name
        :param pulumi.Input[Union['StorageVersioningArgs', 'StorageVersioningArgsDict']] versioning: Versioning configuration
        :param pulumi.Input[builtins.str] virtual_host: Container virtual host
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _StorageState.__new__(_StorageState)

        __props__.__dict__["created_at"] = created_at
        __props__.__dict__["encryption"] = encryption
        __props__.__dict__["limit"] = limit
        __props__.__dict__["marker"] = marker
        __props__.__dict__["name"] = name
        __props__.__dict__["objects"] = objects
        __props__.__dict__["objects_count"] = objects_count
        __props__.__dict__["objects_size"] = objects_size
        __props__.__dict__["owner_id"] = owner_id
        __props__.__dict__["prefix"] = prefix
        __props__.__dict__["region"] = region
        __props__.__dict__["region_name"] = region_name
        __props__.__dict__["replication"] = replication
        __props__.__dict__["service_name"] = service_name
        __props__.__dict__["versioning"] = versioning
        __props__.__dict__["virtual_host"] = virtual_host
        return Storage(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> pulumi.Output[builtins.str]:
        """
        The date and timestamp when the resource was created
        """
        return pulumi.get(self, "created_at")

    @property
    @pulumi.getter
    def encryption(self) -> pulumi.Output['outputs.StorageEncryption']:
        """
        Encryption configuration
        """
        return pulumi.get(self, "encryption")

    @property
    @pulumi.getter
    def limit(self) -> pulumi.Output[builtins.float]:
        """
        Limit the number of objects returned (1000 maximum, defaults to 1000)
        """
        return pulumi.get(self, "limit")

    @property
    @pulumi.getter
    def marker(self) -> pulumi.Output[builtins.str]:
        """
        Key to start with when listing objects
        """
        return pulumi.get(self, "marker")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[builtins.str]:
        """
        Container name
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def objects(self) -> pulumi.Output[Sequence['outputs.StorageObject']]:
        """
        Container objects
        """
        return pulumi.get(self, "objects")

    @property
    @pulumi.getter(name="objectsCount")
    def objects_count(self) -> pulumi.Output[builtins.float]:
        """
        Container total objects count
        """
        return pulumi.get(self, "objects_count")

    @property
    @pulumi.getter(name="objectsSize")
    def objects_size(self) -> pulumi.Output[builtins.float]:
        """
        Container total objects size (bytes)
        """
        return pulumi.get(self, "objects_size")

    @property
    @pulumi.getter(name="ownerId")
    def owner_id(self) -> pulumi.Output[builtins.float]:
        """
        Container owner user ID
        """
        return pulumi.get(self, "owner_id")

    @property
    @pulumi.getter
    def prefix(self) -> pulumi.Output[builtins.str]:
        """
        List objects whose key begins with this prefix
        """
        return pulumi.get(self, "prefix")

    @property
    @pulumi.getter
    def region(self) -> pulumi.Output[builtins.str]:
        """
        Container region
        """
        return pulumi.get(self, "region")

    @property
    @pulumi.getter(name="regionName")
    def region_name(self) -> pulumi.Output[builtins.str]:
        """
        Region name
        """
        return pulumi.get(self, "region_name")

    @property
    @pulumi.getter
    def replication(self) -> pulumi.Output['outputs.StorageReplication']:
        """
        Replication configuration
        """
        return pulumi.get(self, "replication")

    @property
    @pulumi.getter(name="serviceName")
    def service_name(self) -> pulumi.Output[builtins.str]:
        """
        Service name
        """
        return pulumi.get(self, "service_name")

    @property
    @pulumi.getter
    def versioning(self) -> pulumi.Output['outputs.StorageVersioning']:
        """
        Versioning configuration
        """
        return pulumi.get(self, "versioning")

    @property
    @pulumi.getter(name="virtualHost")
    def virtual_host(self) -> pulumi.Output[builtins.str]:
        """
        Container virtual host
        """
        return pulumi.get(self, "virtual_host")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins
from .. import _utilities
import typing
# Export this package's modules as members:
from .ceph_acl import *
from .get_ceph import *
from .get_cloud import *
from .get_nas_ha import *
from .get_nas_ha_partition import *
from .get_server_boots import *
from .get_server_specifications_hardware import *
from .get_server_specifications_network import *
from .nas_ha_partition import *
from .nas_ha_partition_access import *
from .nas_ha_partition_snapshot import *
from .server import *
from .server_networking import *
from .server_reboot_task import *
from .server_reinstall_task import *
from .server_update import *
from ._inputs import *
from . import outputs



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins
import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities

__all__ = ['ZoneRecordArgs', 'ZoneRecord']

@pulumi.input_type
class ZoneRecordArgs:
    def __init__(__self__, *,
                 fieldtype: pulumi.Input[builtins.str],
                 target: pulumi.Input[builtins.str],
                 zone: pulumi.Input[builtins.str],
                 subdomain: Optional[pulumi.Input[builtins.str]] = None,
                 ttl: Optional[pulumi.Input[builtins.int]] = None):
        """
        The set of arguments for constructing a ZoneRecord resource.
        :param pulumi.Input[builtins.str] fieldtype: The type of the record
        :param pulumi.Input[builtins.str] target: The value of the record
        :param pulumi.Input[builtins.str] zone: The domain to add the record to
        :param pulumi.Input[builtins.str] subdomain: The name of the record. It can be an empty string.
        :param pulumi.Input[builtins.int] ttl: The TTL of the record, it shall be >= to 60.
        """
        pulumi.set(__self__, "fieldtype", fieldtype)
        pulumi.set(__self__, "target", target)
        pulumi.set(__self__, "zone", zone)
        if subdomain is not None:
            pulumi.set(__self__, "subdomain", subdomain)
        if ttl is not None:
            pulumi.set(__self__, "ttl", ttl)

    @property
    @pulumi.getter
    def fieldtype(self) -> pulumi.Input[builtins.str]:
        """
        The type of the record
        """
        return pulumi.get(self, "fieldtype")

    @fieldtype.setter
    def fieldtype(self, value: pulumi.Input[builtins.str]):
        pulumi.set(self, "fieldtype", value)

    @property
    @pulumi.getter
    def target(self) -> pulumi.Input[builtins.str]:
        """
        The value of the record
        """
        return pulumi.get(self, "target")

    @target.setter
    def target(self, value: pulumi.Input[builtins.str]):
        pulumi.set(self, "target", value)

    @property
    @pulumi.getter
    def zone(self) -> pulumi.Input[builtins.str]:
        """
        The domain to add the record to
        """
        return pulumi.get(self, "zone")

    @zone.setter
    def zone(self, value: pulumi.Input[builtins.str]):
        pulumi.set(self, "zone", value)

    @property
    @pulumi.getter
    def subdomain(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        The name of the record. It can be an empty string.
        """
        return pulumi.get(self, "subdomain")

    @subdomain.setter
    def subdomain(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "subdomain", value)

    @property
    @pulumi.getter
    def ttl(self) -> Optional[pulumi.Input[builtins.int]]:
        """
        The TTL of the record, it shall be >= to 60.
        """
        return pulumi.get(self, "ttl")

    @ttl.setter
    def ttl(self, value: Optional[pulumi.Input[builtins.int]]):
        pulumi.set(self, "ttl", value)


@pulumi.input_type
class _ZoneRecordState:
    def __init__(__self__, *,
                 fieldtype: Optional[pulumi.Input[builtins.str]] = None,
                 subdomain: Optional[pulumi.Input[builtins.str]] = None,
                 target: Optional[pulumi.Input[builtins.str]] = None,
                 ttl: Optional[pulumi.Input[builtins.int]] = None,
                 zone: Optional[pulumi.Input[builtins.str]] = None):
        """
        Input properties used for looking up and filtering ZoneRecord resources.
        :param pulumi.Input[builtins.str] fieldtype: The type of the record
        :param pulumi.Input[builtins.str] subdomain: The name of the record. It can be an empty string.
        :param pulumi.Input[builtins.str] target: The value of the record
        :param pulumi.Input[builtins.int] ttl: The TTL of the record, it shall be >= to 60.
        :param pulumi.Input[builtins.str] zone: The domain to add the record to
        """
        if fieldtype is not None:
            pulumi.set(__self__, "fieldtype", fieldtype)
        if subdomain is not None:
            pulumi.set(__self__, "subdomain", subdomain)
        if target is not None:
            pulumi.set(__self__, "target", target)
        if ttl is not None:
            pulumi.set(__self__, "ttl", ttl)
        if zone is not None:
            pulumi.set(__self__, "zone", zone)

    @property
    @pulumi.getter
    def fieldtype(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        The type of the record
        """
        return pulumi.get(self, "fieldtype")

    @fieldtype.setter
    def fieldtype(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "fieldtype", value)

    @property
    @pulumi.getter
    def subdomain(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        The name of the record. It can be an empty string.
        """
        return pulumi.get(self, "subdomain")

    @subdomain.setter
    def subdomain(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "subdomain", value)

    @property
    @pulumi.getter
    def target(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        The value of the record
        """
        return pulumi.get(self, "target")

    @target.setter
    def target(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "target", value)

    @property
    @pulumi.getter
    def ttl(self) -> Optional[pulumi.Input[builtins.int]]:
        """
        The TTL of the record, it shall be >= to 60.
        """
        return pulumi.get(self, "ttl")

    @ttl.setter
    def ttl(self, value: Optional[pulumi.Input[builtins.int]]):
        pulumi.set(self, "ttl", value)

    @property
    @pulumi.getter
    def zone(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        The domain to add the record to
        """
        return pulumi.get(self, "zone")

    @zone.setter
    def zone(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "zone", value)


class ZoneRecord(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 fieldtype: Optional[pulumi.Input[builtins.str]] = None,
                 subdomain: Optional[pulumi.Input[builtins.str]] = None,
                 target: Optional[pulumi.Input[builtins.str]] = None,
                 ttl: Optional[pulumi.Input[builtins.int]] = None,
                 zone: Optional[pulumi.Input[builtins.str]] = None,
                 __props__=None):
        """
        ## Example Usage

        ```python
        import pulumi
        import pulumi_ovh as ovh

        # Add a record to a sub-domain
        test = ovh.domain.ZoneRecord("test",
            zone="testdemo.ovh",
            subdomain="test",
            fieldtype="A",
            ttl=3600,
            target="0.0.0.0")
        ```

        ## Import

        OVHcloud domain zone record can be imported using the `id`, which can be retrieved by using [OVH API portal](https://api.ovh.com/console/#/domain/zone/%7BzoneName%7D/record~GET), and the `zone`, separated by "." E.g.,

        bash

        ```sh
        $ pulumi import ovh:Domain/zoneRecord:ZoneRecord test id.zone
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[builtins.str] fieldtype: The type of the record
        :param pulumi.Input[builtins.str] subdomain: The name of the record. It can be an empty string.
        :param pulumi.Input[builtins.str] target: The value of the record
        :param pulumi.Input[builtins.int] ttl: The TTL of the record, it shall be >= to 60.
        :param pulumi.Input[builtins.str] zone: The domain to add the record to
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: ZoneRecordArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        ## Example Usage

        ```python
        import pulumi
        import pulumi_ovh as ovh

        # Add a record to a sub-domain
        test = ovh.domain.ZoneRecord("test",
            zone="testdemo.ovh",
            subdomain="test",
            fieldtype="A",
            ttl=3600,
            target="0.0.0.0")
        ```

        ## Import

        OVHcloud domain zone record can be imported using the `id`, which can be retrieved by using [OVH API portal](https://api.ovh.com/console/#/domain/zone/%7BzoneName%7D/record~GET), and the `zone`, separated by "." E.g.,

        bash

        ```sh
        $ pulumi import ovh:Domain/zoneRecord:ZoneRecord test id.zone
        ```

        :param str resource_name: The name of the resource.
        :param ZoneRecordArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ZoneRecordArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 fieldtype: Optional[pulumi.Input[builtins.str]] = None,
                 subdomain: Optional[pulumi.Input[builtins.str]] = None,
                 target: Optional[pulumi.Input[builtins.str]] = None,
                 ttl: Optional[pulumi.Input[builtins.int]] = None,
                 zone: Optional[pulumi.Input[builtins.str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ZoneRecordArgs.__new__(ZoneRecordArgs)

            if fieldtype is None and not opts.urn:
                raise TypeError("Missing required property 'fieldtype'")
            __props__.__dict__["fieldtype"] = fieldtype
            __props__.__dict__["subdomain"] = subdomain
            if target is None and not opts.urn:
                raise TypeError("Missing required property 'target'")
            __props__.__dict__["target"] = target
            __props__.__dict__["ttl"] = ttl
            if zone is None and not opts.urn:
                raise TypeError("Missing required property 'zone'")
            __props__.__dict__["zone"] = zone
        super(ZoneRecord, __self__).__init__(
            'ovh:Domain/zoneRecord:ZoneRecord',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            fieldtype: Optional[pulumi.Input[builtins.str]] = None,
            subdomain: Optional[pulumi.Input[builtins.str]] = None,
            target: Optional[pulumi.Input[builtins.str]] = None,
            ttl: Optional[pulumi.Input[builtins.int]] = None,
            zone: Optional[pulumi.Input[builtins.str]] = None) -> 'ZoneRecord':
        """
        Get an existing ZoneRecord resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[builtins.str] fieldtype: The type of the record
        :param pulumi.Input[builtins.str] subdomain: The name of the record. It can be an empty string.
        :param pulumi.Input[builtins.str] target: The value of the record
        :param pulumi.Input[builtins.int] ttl: The TTL of the record, it shall be >= to 60.
        :param pulumi.Input[builtins.str] zone: The domain to add the record to
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _ZoneRecordState.__new__(_ZoneRecordState)

        __props__.__dict__["fieldtype"] = fieldtype
        __props__.__dict__["subdomain"] = subdomain
        __props__.__dict__["target"] = target
        __props__.__dict__["ttl"] = ttl
        __props__.__dict__["zone"] = zone
        return ZoneRecord(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def fieldtype(self) -> pulumi.Output[builtins.str]:
        """
        The type of the record
        """
        return pulumi.get(self, "fieldtype")

    @property
    @pulumi.getter
    def subdomain(self) -> pulumi.Output[Optional[builtins.str]]:
        """
        The name of the record. It can be an empty string.
        """
        return pulumi.get(self, "subdomain")

    @property
    @pulumi.getter
    def target(self) -> pulumi.Output[builtins.str]:
        """
        The value of the record
        """
        return pulumi.get(self, "target")

    @property
    @pulumi.getter
    def ttl(self) -> pulumi.Output[Optional[builtins.int]]:
        """
        The TTL of the record, it shall be >= to 60.
        """
        return pulumi.get(self, "ttl")

    @property
    @pulumi.getter
    def zone(self) -> pulumi.Output[builtins.str]:
        """
        The domain to add the record to
        """
        return pulumi.get(self, "zone")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins
import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities
from . import outputs

__all__ = [
    'GetOkmsResourceResult',
    'AwaitableGetOkmsResourceResult',
    'get_okms_resource',
    'get_okms_resource_output',
]

@pulumi.output_type
class GetOkmsResourceResult:
    """
    A collection of values returned by getOkmsResource.
    """
    def __init__(__self__, iam=None, id=None, kmip_endpoint=None, public_ca=None, region=None, rest_endpoint=None, swagger_endpoint=None):
        if iam and not isinstance(iam, dict):
            raise TypeError("Expected argument 'iam' to be a dict")
        pulumi.set(__self__, "iam", iam)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if kmip_endpoint and not isinstance(kmip_endpoint, str):
            raise TypeError("Expected argument 'kmip_endpoint' to be a str")
        pulumi.set(__self__, "kmip_endpoint", kmip_endpoint)
        if public_ca and not isinstance(public_ca, str):
            raise TypeError("Expected argument 'public_ca' to be a str")
        pulumi.set(__self__, "public_ca", public_ca)
        if region and not isinstance(region, str):
            raise TypeError("Expected argument 'region' to be a str")
        pulumi.set(__self__, "region", region)
        if rest_endpoint and not isinstance(rest_endpoint, str):
            raise TypeError("Expected argument 'rest_endpoint' to be a str")
        pulumi.set(__self__, "rest_endpoint", rest_endpoint)
        if swagger_endpoint and not isinstance(swagger_endpoint, str):
            raise TypeError("Expected argument 'swagger_endpoint' to be a str")
        pulumi.set(__self__, "swagger_endpoint", swagger_endpoint)

    @property
    @pulumi.getter
    def iam(self) -> 'outputs.GetOkmsResourceIamResult':
        """
        (Attributes) IAM resource metadata (see below for nested schema)
        """
        return pulumi.get(self, "iam")

    @property
    @pulumi.getter
    def id(self) -> builtins.str:
        """
        (String) Unique identifier of the resource
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="kmipEndpoint")
    def kmip_endpoint(self) -> builtins.str:
        """
        (String) KMS kmip API endpoint
        """
        return pulumi.get(self, "kmip_endpoint")

    @property
    @pulumi.getter(name="publicCa")
    def public_ca(self) -> builtins.str:
        """
        (String) KMS public CA (Certificate Authority)
        """
        return pulumi.get(self, "public_ca")

    @property
    @pulumi.getter
    def region(self) -> builtins.str:
        """
        (String) Region
        """
        return pulumi.get(self, "region")

    @property
    @pulumi.getter(name="restEndpoint")
    def rest_endpoint(self) -> builtins.str:
        """
        (String) KMS rest API endpoint
        """
        return pulumi.get(self, "rest_endpoint")

    @property
    @pulumi.getter(name="swaggerEndpoint")
    def swagger_endpoint(self) -> builtins.str:
        """
        (String) KMS rest API swagger UI
        """
        return pulumi.get(self, "swagger_endpoint")


class AwaitableGetOkmsResourceResult(GetOkmsResourceResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetOkmsResourceResult(
            iam=self.iam,
            id=self.id,
            kmip_endpoint=self.kmip_endpoint,
            public_ca=self.public_ca,
            region=self.region,
            rest_endpoint=self.rest_endpoint,
            swagger_endpoint=self.swagger_endpoint)


def get_okms_resource(id: Optional[builtins.str] = None,
                      opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetOkmsResourceResult:
    """
    Use this data source to retrieve information about a KMS associated with this account

    ## Example Usage

    ```python
    import pulumi
    import pulumi_ovh as ovh

    kms = ovh.Okms.get_okms_resource(id="xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx")
    ```


    :param builtins.str id: Should be set to the ID of your KMS
    """
    __args__ = dict()
    __args__['id'] = id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('ovh:Okms/getOkmsResource:getOkmsResource', __args__, opts=opts, typ=GetOkmsResourceResult).value

    return AwaitableGetOkmsResourceResult(
        iam=pulumi.get(__ret__, 'iam'),
        id=pulumi.get(__ret__, 'id'),
        kmip_endpoint=pulumi.get(__ret__, 'kmip_endpoint'),
        public_ca=pulumi.get(__ret__, 'public_ca'),
        region=pulumi.get(__ret__, 'region'),
        rest_endpoint=pulumi.get(__ret__, 'rest_endpoint'),
        swagger_endpoint=pulumi.get(__ret__, 'swagger_endpoint'))
def get_okms_resource_output(id: Optional[pulumi.Input[builtins.str]] = None,
                             opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetOkmsResourceResult]:
    """
    Use this data source to retrieve information about a KMS associated with this account

    ## Example Usage

    ```python
    import pulumi
    import pulumi_ovh as ovh

    kms = ovh.Okms.get_okms_resource(id="xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx")
    ```


    :param builtins.str id: Should be set to the ID of your KMS
    """
    __args__ = dict()
    __args__['id'] = id
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('ovh:Okms/getOkmsResource:getOkmsResource', __args__, opts=opts, typ=GetOkmsResourceResult)
    return __ret__.apply(lambda __response__: GetOkmsResourceResult(
        iam=pulumi.get(__response__, 'iam'),
        id=pulumi.get(__response__, 'id'),
        kmip_endpoint=pulumi.get(__response__, 'kmip_endpoint'),
        public_ca=pulumi.get(__response__, 'public_ca'),
        region=pulumi.get(__response__, 'region'),
        rest_endpoint=pulumi.get(__response__, 'rest_endpoint'),
        swagger_endpoint=pulumi.get(__response__, 'swagger_endpoint')))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins
import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = ['InstanceArgs', 'Instance']

@pulumi.input_type
class InstanceArgs:
    def __init__(__self__, *,
                 billing_period: pulumi.Input[builtins.str],
                 boot_from: pulumi.Input['InstanceBootFromArgs'],
                 flavor: pulumi.Input['InstanceFlavorArgs'],
                 network: pulumi.Input['InstanceNetworkArgs'],
                 region: pulumi.Input[builtins.str],
                 service_name: pulumi.Input[builtins.str],
                 auto_backup: Optional[pulumi.Input['InstanceAutoBackupArgs']] = None,
                 availability_zone: Optional[pulumi.Input[builtins.str]] = None,
                 bulk: Optional[pulumi.Input[builtins.int]] = None,
                 group: Optional[pulumi.Input['InstanceGroupArgs']] = None,
                 name: Optional[pulumi.Input[builtins.str]] = None,
                 ssh_key: Optional[pulumi.Input['InstanceSshKeyArgs']] = None,
                 ssh_key_create: Optional[pulumi.Input['InstanceSshKeyCreateArgs']] = None,
                 user_data: Optional[pulumi.Input[builtins.str]] = None):
        """
        The set of arguments for constructing a Instance resource.
        :param pulumi.Input[builtins.str] billing_period: Billing period - hourly or monthly
        :param pulumi.Input['InstanceBootFromArgs'] boot_from: Boot the instance from an image or a volume
        :param pulumi.Input['InstanceFlavorArgs'] flavor: Flavor information
        :param pulumi.Input['InstanceNetworkArgs'] network: Create network interfaces
        :param pulumi.Input[builtins.str] region: Instance region
        :param pulumi.Input[builtins.str] service_name: The id of the public cloud project. If omitted, the `OVH_CLOUD_PROJECT_SERVICE` environment variable is used
        :param pulumi.Input['InstanceAutoBackupArgs'] auto_backup: Create an autobackup workflow after instance start up.
        :param pulumi.Input[builtins.str] availability_zone: The availability zone where the instance will be created
        :param pulumi.Input[builtins.int] bulk: Create multiple instances
        :param pulumi.Input['InstanceGroupArgs'] group: Start instance in group
        :param pulumi.Input[builtins.str] name: Instance name
        :param pulumi.Input['InstanceSshKeyArgs'] ssh_key: Existing SSH Keypair
        :param pulumi.Input['InstanceSshKeyCreateArgs'] ssh_key_create: Add existing SSH Key pair into your Public Cloud project and link it to the instance
        :param pulumi.Input[builtins.str] user_data: Configuration information or scripts to use upon launch
        """
        pulumi.set(__self__, "billing_period", billing_period)
        pulumi.set(__self__, "boot_from", boot_from)
        pulumi.set(__self__, "flavor", flavor)
        pulumi.set(__self__, "network", network)
        pulumi.set(__self__, "region", region)
        pulumi.set(__self__, "service_name", service_name)
        if auto_backup is not None:
            pulumi.set(__self__, "auto_backup", auto_backup)
        if availability_zone is not None:
            pulumi.set(__self__, "availability_zone", availability_zone)
        if bulk is not None:
            pulumi.set(__self__, "bulk", bulk)
        if group is not None:
            pulumi.set(__self__, "group", group)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if ssh_key is not None:
            pulumi.set(__self__, "ssh_key", ssh_key)
        if ssh_key_create is not None:
            pulumi.set(__self__, "ssh_key_create", ssh_key_create)
        if user_data is not None:
            pulumi.set(__self__, "user_data", user_data)

    @property
    @pulumi.getter(name="billingPeriod")
    def billing_period(self) -> pulumi.Input[builtins.str]:
        """
        Billing period - hourly or monthly
        """
        return pulumi.get(self, "billing_period")

    @billing_period.setter
    def billing_period(self, value: pulumi.Input[builtins.str]):
        pulumi.set(self, "billing_period", value)

    @property
    @pulumi.getter(name="bootFrom")
    def boot_from(self) -> pulumi.Input['InstanceBootFromArgs']:
        """
        Boot the instance from an image or a volume
        """
        return pulumi.get(self, "boot_from")

    @boot_from.setter
    def boot_from(self, value: pulumi.Input['InstanceBootFromArgs']):
        pulumi.set(self, "boot_from", value)

    @property
    @pulumi.getter
    def flavor(self) -> pulumi.Input['InstanceFlavorArgs']:
        """
        Flavor information
        """
        return pulumi.get(self, "flavor")

    @flavor.setter
    def flavor(self, value: pulumi.Input['InstanceFlavorArgs']):
        pulumi.set(self, "flavor", value)

    @property
    @pulumi.getter
    def network(self) -> pulumi.Input['InstanceNetworkArgs']:
        """
        Create network interfaces
        """
        return pulumi.get(self, "network")

    @network.setter
    def network(self, value: pulumi.Input['InstanceNetworkArgs']):
        pulumi.set(self, "network", value)

    @property
    @pulumi.getter
    def region(self) -> pulumi.Input[builtins.str]:
        """
        Instance region
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: pulumi.Input[builtins.str]):
        pulumi.set(self, "region", value)

    @property
    @pulumi.getter(name="serviceName")
    def service_name(self) -> pulumi.Input[builtins.str]:
        """
        The id of the public cloud project. If omitted, the `OVH_CLOUD_PROJECT_SERVICE` environment variable is used
        """
        return pulumi.get(self, "service_name")

    @service_name.setter
    def service_name(self, value: pulumi.Input[builtins.str]):
        pulumi.set(self, "service_name", value)

    @property
    @pulumi.getter(name="autoBackup")
    def auto_backup(self) -> Optional[pulumi.Input['InstanceAutoBackupArgs']]:
        """
        Create an autobackup workflow after instance start up.
        """
        return pulumi.get(self, "auto_backup")

    @auto_backup.setter
    def auto_backup(self, value: Optional[pulumi.Input['InstanceAutoBackupArgs']]):
        pulumi.set(self, "auto_backup", value)

    @property
    @pulumi.getter(name="availabilityZone")
    def availability_zone(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        The availability zone where the instance will be created
        """
        return pulumi.get(self, "availability_zone")

    @availability_zone.setter
    def availability_zone(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "availability_zone", value)

    @property
    @pulumi.getter
    def bulk(self) -> Optional[pulumi.Input[builtins.int]]:
        """
        Create multiple instances
        """
        return pulumi.get(self, "bulk")

    @bulk.setter
    def bulk(self, value: Optional[pulumi.Input[builtins.int]]):
        pulumi.set(self, "bulk", value)

    @property
    @pulumi.getter
    def group(self) -> Optional[pulumi.Input['InstanceGroupArgs']]:
        """
        Start instance in group
        """
        return pulumi.get(self, "group")

    @group.setter
    def group(self, value: Optional[pulumi.Input['InstanceGroupArgs']]):
        pulumi.set(self, "group", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Instance name
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter(name="sshKey")
    def ssh_key(self) -> Optional[pulumi.Input['InstanceSshKeyArgs']]:
        """
        Existing SSH Keypair
        """
        return pulumi.get(self, "ssh_key")

    @ssh_key.setter
    def ssh_key(self, value: Optional[pulumi.Input['InstanceSshKeyArgs']]):
        pulumi.set(self, "ssh_key", value)

    @property
    @pulumi.getter(name="sshKeyCreate")
    def ssh_key_create(self) -> Optional[pulumi.Input['InstanceSshKeyCreateArgs']]:
        """
        Add existing SSH Key pair into your Public Cloud project and link it to the instance
        """
        return pulumi.get(self, "ssh_key_create")

    @ssh_key_create.setter
    def ssh_key_create(self, value: Optional[pulumi.Input['InstanceSshKeyCreateArgs']]):
        pulumi.set(self, "ssh_key_create", value)

    @property
    @pulumi.getter(name="userData")
    def user_data(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Configuration information or scripts to use upon launch
        """
        return pulumi.get(self, "user_data")

    @user_data.setter
    def user_data(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "user_data", value)


@pulumi.input_type
class _InstanceState:
    def __init__(__self__, *,
                 addresses: Optional[pulumi.Input[Sequence[pulumi.Input['InstanceAddressArgs']]]] = None,
                 attached_volumes: Optional[pulumi.Input[Sequence[pulumi.Input['InstanceAttachedVolumeArgs']]]] = None,
                 auto_backup: Optional[pulumi.Input['InstanceAutoBackupArgs']] = None,
                 availability_zone: Optional[pulumi.Input[builtins.str]] = None,
                 billing_period: Optional[pulumi.Input[builtins.str]] = None,
                 boot_from: Optional[pulumi.Input['InstanceBootFromArgs']] = None,
                 bulk: Optional[pulumi.Input[builtins.int]] = None,
                 flavor: Optional[pulumi.Input['InstanceFlavorArgs']] = None,
                 flavor_id: Optional[pulumi.Input[builtins.str]] = None,
                 flavor_name: Optional[pulumi.Input[builtins.str]] = None,
                 group: Optional[pulumi.Input['InstanceGroupArgs']] = None,
                 image_id: Optional[pulumi.Input[builtins.str]] = None,
                 name: Optional[pulumi.Input[builtins.str]] = None,
                 network: Optional[pulumi.Input['InstanceNetworkArgs']] = None,
                 region: Optional[pulumi.Input[builtins.str]] = None,
                 service_name: Optional[pulumi.Input[builtins.str]] = None,
                 ssh_key: Optional[pulumi.Input['InstanceSshKeyArgs']] = None,
                 ssh_key_create: Optional[pulumi.Input['InstanceSshKeyCreateArgs']] = None,
                 task_state: Optional[pulumi.Input[builtins.str]] = None,
                 user_data: Optional[pulumi.Input[builtins.str]] = None):
        """
        Input properties used for looking up and filtering Instance resources.
        :param pulumi.Input[Sequence[pulumi.Input['InstanceAddressArgs']]] addresses: Instance IP addresses
        :param pulumi.Input[Sequence[pulumi.Input['InstanceAttachedVolumeArgs']]] attached_volumes: Volumes attached to the instance
        :param pulumi.Input['InstanceAutoBackupArgs'] auto_backup: Create an autobackup workflow after instance start up.
        :param pulumi.Input[builtins.str] availability_zone: The availability zone where the instance will be created
        :param pulumi.Input[builtins.str] billing_period: Billing period - hourly or monthly
        :param pulumi.Input['InstanceBootFromArgs'] boot_from: Boot the instance from an image or a volume
        :param pulumi.Input[builtins.int] bulk: Create multiple instances
        :param pulumi.Input['InstanceFlavorArgs'] flavor: Flavor information
        :param pulumi.Input[builtins.str] flavor_id: Flavor id
        :param pulumi.Input[builtins.str] flavor_name: Flavor name
        :param pulumi.Input['InstanceGroupArgs'] group: Start instance in group
        :param pulumi.Input[builtins.str] image_id: Image id
        :param pulumi.Input[builtins.str] name: Instance name
        :param pulumi.Input['InstanceNetworkArgs'] network: Create network interfaces
        :param pulumi.Input[builtins.str] region: Instance region
        :param pulumi.Input[builtins.str] service_name: The id of the public cloud project. If omitted, the `OVH_CLOUD_PROJECT_SERVICE` environment variable is used
        :param pulumi.Input['InstanceSshKeyArgs'] ssh_key: Existing SSH Keypair
        :param pulumi.Input['InstanceSshKeyCreateArgs'] ssh_key_create: Add existing SSH Key pair into your Public Cloud project and link it to the instance
        :param pulumi.Input[builtins.str] task_state: Instance task state
        :param pulumi.Input[builtins.str] user_data: Configuration information or scripts to use upon launch
        """
        if addresses is not None:
            pulumi.set(__self__, "addresses", addresses)
        if attached_volumes is not None:
            pulumi.set(__self__, "attached_volumes", attached_volumes)
        if auto_backup is not None:
            pulumi.set(__self__, "auto_backup", auto_backup)
        if availability_zone is not None:
            pulumi.set(__self__, "availability_zone", availability_zone)
        if billing_period is not None:
            pulumi.set(__self__, "billing_period", billing_period)
        if boot_from is not None:
            pulumi.set(__self__, "boot_from", boot_from)
        if bulk is not None:
            pulumi.set(__self__, "bulk", bulk)
        if flavor is not None:
            pulumi.set(__self__, "flavor", flavor)
        if flavor_id is not None:
            pulumi.set(__self__, "flavor_id", flavor_id)
        if flavor_name is not None:
            pulumi.set(__self__, "flavor_name", flavor_name)
        if group is not None:
            pulumi.set(__self__, "group", group)
        if image_id is not None:
            pulumi.set(__self__, "image_id", image_id)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if network is not None:
            pulumi.set(__self__, "network", network)
        if region is not None:
            pulumi.set(__self__, "region", region)
        if service_name is not None:
            pulumi.set(__self__, "service_name", service_name)
        if ssh_key is not None:
            pulumi.set(__self__, "ssh_key", ssh_key)
        if ssh_key_create is not None:
            pulumi.set(__self__, "ssh_key_create", ssh_key_create)
        if task_state is not None:
            pulumi.set(__self__, "task_state", task_state)
        if user_data is not None:
            pulumi.set(__self__, "user_data", user_data)

    @property
    @pulumi.getter
    def addresses(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['InstanceAddressArgs']]]]:
        """
        Instance IP addresses
        """
        return pulumi.get(self, "addresses")

    @addresses.setter
    def addresses(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['InstanceAddressArgs']]]]):
        pulumi.set(self, "addresses", value)

    @property
    @pulumi.getter(name="attachedVolumes")
    def attached_volumes(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['InstanceAttachedVolumeArgs']]]]:
        """
        Volumes attached to the instance
        """
        return pulumi.get(self, "attached_volumes")

    @attached_volumes.setter
    def attached_volumes(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['InstanceAttachedVolumeArgs']]]]):
        pulumi.set(self, "attached_volumes", value)

    @property
    @pulumi.getter(name="autoBackup")
    def auto_backup(self) -> Optional[pulumi.Input['InstanceAutoBackupArgs']]:
        """
        Create an autobackup workflow after instance start up.
        """
        return pulumi.get(self, "auto_backup")

    @auto_backup.setter
    def auto_backup(self, value: Optional[pulumi.Input['InstanceAutoBackupArgs']]):
        pulumi.set(self, "auto_backup", value)

    @property
    @pulumi.getter(name="availabilityZone")
    def availability_zone(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        The availability zone where the instance will be created
        """
        return pulumi.get(self, "availability_zone")

    @availability_zone.setter
    def availability_zone(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "availability_zone", value)

    @property
    @pulumi.getter(name="billingPeriod")
    def billing_period(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Billing period - hourly or monthly
        """
        return pulumi.get(self, "billing_period")

    @billing_period.setter
    def billing_period(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "billing_period", value)

    @property
    @pulumi.getter(name="bootFrom")
    def boot_from(self) -> Optional[pulumi.Input['InstanceBootFromArgs']]:
        """
        Boot the instance from an image or a volume
        """
        return pulumi.get(self, "boot_from")

    @boot_from.setter
    def boot_from(self, value: Optional[pulumi.Input['InstanceBootFromArgs']]):
        pulumi.set(self, "boot_from", value)

    @property
    @pulumi.getter
    def bulk(self) -> Optional[pulumi.Input[builtins.int]]:
        """
        Create multiple instances
        """
        return pulumi.get(self, "bulk")

    @bulk.setter
    def bulk(self, value: Optional[pulumi.Input[builtins.int]]):
        pulumi.set(self, "bulk", value)

    @property
    @pulumi.getter
    def flavor(self) -> Optional[pulumi.Input['InstanceFlavorArgs']]:
        """
        Flavor information
        """
        return pulumi.get(self, "flavor")

    @flavor.setter
    def flavor(self, value: Optional[pulumi.Input['InstanceFlavorArgs']]):
        pulumi.set(self, "flavor", value)

    @property
    @pulumi.getter(name="flavorId")
    def flavor_id(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Flavor id
        """
        return pulumi.get(self, "flavor_id")

    @flavor_id.setter
    def flavor_id(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "flavor_id", value)

    @property
    @pulumi.getter(name="flavorName")
    def flavor_name(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Flavor name
        """
        return pulumi.get(self, "flavor_name")

    @flavor_name.setter
    def flavor_name(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "flavor_name", value)

    @property
    @pulumi.getter
    def group(self) -> Optional[pulumi.Input['InstanceGroupArgs']]:
        """
        Start instance in group
        """
        return pulumi.get(self, "group")

    @group.setter
    def group(self, value: Optional[pulumi.Input['InstanceGroupArgs']]):
        pulumi.set(self, "group", value)

    @property
    @pulumi.getter(name="imageId")
    def image_id(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Image id
        """
        return pulumi.get(self, "image_id")

    @image_id.setter
    def image_id(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "image_id", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Instance name
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def network(self) -> Optional[pulumi.Input['InstanceNetworkArgs']]:
        """
        Create network interfaces
        """
        return pulumi.get(self, "network")

    @network.setter
    def network(self, value: Optional[pulumi.Input['InstanceNetworkArgs']]):
        pulumi.set(self, "network", value)

    @property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Instance region
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "region", value)

    @property
    @pulumi.getter(name="serviceName")
    def service_name(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        The id of the public cloud project. If omitted, the `OVH_CLOUD_PROJECT_SERVICE` environment variable is used
        """
        return pulumi.get(self, "service_name")

    @service_name.setter
    def service_name(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "service_name", value)

    @property
    @pulumi.getter(name="sshKey")
    def ssh_key(self) -> Optional[pulumi.Input['InstanceSshKeyArgs']]:
        """
        Existing SSH Keypair
        """
        return pulumi.get(self, "ssh_key")

    @ssh_key.setter
    def ssh_key(self, value: Optional[pulumi.Input['InstanceSshKeyArgs']]):
        pulumi.set(self, "ssh_key", value)

    @property
    @pulumi.getter(name="sshKeyCreate")
    def ssh_key_create(self) -> Optional[pulumi.Input['InstanceSshKeyCreateArgs']]:
        """
        Add existing SSH Key pair into your Public Cloud project and link it to the instance
        """
        return pulumi.get(self, "ssh_key_create")

    @ssh_key_create.setter
    def ssh_key_create(self, value: Optional[pulumi.Input['InstanceSshKeyCreateArgs']]):
        pulumi.set(self, "ssh_key_create", value)

    @property
    @pulumi.getter(name="taskState")
    def task_state(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Instance task state
        """
        return pulumi.get(self, "task_state")

    @task_state.setter
    def task_state(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "task_state", value)

    @property
    @pulumi.getter(name="userData")
    def user_data(self) -> Optional[pulumi.Input[builtins.str]]:
        """
        Configuration information or scripts to use upon launch
        """
        return pulumi.get(self, "user_data")

    @user_data.setter
    def user_data(self, value: Optional[pulumi.Input[builtins.str]]):
        pulumi.set(self, "user_data", value)


class Instance(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 auto_backup: Optional[pulumi.Input[Union['InstanceAutoBackupArgs', 'InstanceAutoBackupArgsDict']]] = None,
                 availability_zone: Optional[pulumi.Input[builtins.str]] = None,
                 billing_period: Optional[pulumi.Input[builtins.str]] = None,
                 boot_from: Optional[pulumi.Input[Union['InstanceBootFromArgs', 'InstanceBootFromArgsDict']]] = None,
                 bulk: Optional[pulumi.Input[builtins.int]] = None,
                 flavor: Optional[pulumi.Input[Union['InstanceFlavorArgs', 'InstanceFlavorArgsDict']]] = None,
                 group: Optional[pulumi.Input[Union['InstanceGroupArgs', 'InstanceGroupArgsDict']]] = None,
                 name: Optional[pulumi.Input[builtins.str]] = None,
                 network: Optional[pulumi.Input[Union['InstanceNetworkArgs', 'InstanceNetworkArgsDict']]] = None,
                 region: Optional[pulumi.Input[builtins.str]] = None,
                 service_name: Optional[pulumi.Input[builtins.str]] = None,
                 ssh_key: Optional[pulumi.Input[Union['InstanceSshKeyArgs', 'InstanceSshKeyArgsDict']]] = None,
                 ssh_key_create: Optional[pulumi.Input[Union['InstanceSshKeyCreateArgs', 'InstanceSshKeyCreateArgsDict']]] = None,
                 user_data: Optional[pulumi.Input[builtins.str]] = None,
                 __props__=None):
        """
        **This resource uses a Beta API** Creates an instance associated with a public cloud project.

        ## Example Usage

        Create a instance.

        ```python
        import pulumi
        import pulumi_ovh as ovh

        instance = ovh.cloud_project.Instance("instance",
            service_name="XXX",
            region="RRRR",
            billing_period="hourly",
            boot_from={
                "image_id": "UUID",
            },
            flavor={
                "flavor_id": "UUID",
            },
            name="instance name",
            ssh_key={
                "name": "sshname",
            },
            network={
                "public": True,
            })
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Union['InstanceAutoBackupArgs', 'InstanceAutoBackupArgsDict']] auto_backup: Create an autobackup workflow after instance start up.
        :param pulumi.Input[builtins.str] availability_zone: The availability zone where the instance will be created
        :param pulumi.Input[builtins.str] billing_period: Billing period - hourly or monthly
        :param pulumi.Input[Union['InstanceBootFromArgs', 'InstanceBootFromArgsDict']] boot_from: Boot the instance from an image or a volume
        :param pulumi.Input[builtins.int] bulk: Create multiple instances
        :param pulumi.Input[Union['InstanceFlavorArgs', 'InstanceFlavorArgsDict']] flavor: Flavor information
        :param pulumi.Input[Union['InstanceGroupArgs', 'InstanceGroupArgsDict']] group: Start instance in group
        :param pulumi.Input[builtins.str] name: Instance name
        :param pulumi.Input[Union['InstanceNetworkArgs', 'InstanceNetworkArgsDict']] network: Create network interfaces
        :param pulumi.Input[builtins.str] region: Instance region
        :param pulumi.Input[builtins.str] service_name: The id of the public cloud project. If omitted, the `OVH_CLOUD_PROJECT_SERVICE` environment variable is used
        :param pulumi.Input[Union['InstanceSshKeyArgs', 'InstanceSshKeyArgsDict']] ssh_key: Existing SSH Keypair
        :param pulumi.Input[Union['InstanceSshKeyCreateArgs', 'InstanceSshKeyCreateArgsDict']] ssh_key_create: Add existing SSH Key pair into your Public Cloud project and link it to the instance
        :param pulumi.Input[builtins.str] user_data: Configuration information or scripts to use upon launch
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: InstanceArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        **This resource uses a Beta API** Creates an instance associated with a public cloud project.

        ## Example Usage

        Create a instance.

        ```python
        import pulumi
        import pulumi_ovh as ovh

        instance = ovh.cloud_project.Instance("instance",
            service_name="XXX",
            region="RRRR",
            billing_period="hourly",
            boot_from={
                "image_id": "UUID",
            },
            flavor={
                "flavor_id": "UUID",
            },
            name="instance name",
            ssh_key={
                "name": "sshname",
            },
            network={
                "public": True,
            })
        ```

        :param str resource_name: The name of the resource.
        :param InstanceArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(InstanceArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 auto_backup: Optional[pulumi.Input[Union['InstanceAutoBackupArgs', 'InstanceAutoBackupArgsDict']]] = None,
                 availability_zone: Optional[pulumi.Input[builtins.str]] = None,
                 billing_period: Optional[pulumi.Input[builtins.str]] = None,
                 boot_from: Optional[pulumi.Input[Union['InstanceBootFromArgs', 'InstanceBootFromArgsDict']]] = None,
                 bulk: Optional[pulumi.Input[builtins.int]] = None,
                 flavor: Optional[pulumi.Input[Union['InstanceFlavorArgs', 'InstanceFlavorArgsDict']]] = None,
                 group: Optional[pulumi.Input[Union['InstanceGroupArgs', 'InstanceGroupArgsDict']]] = None,
                 name: Optional[pulumi.Input[builtins.str]] = None,
                 network: Optional[pulumi.Input[Union['InstanceNetworkArgs', 'InstanceNetworkArgsDict']]] = None,
                 region: Optional[pulumi.Input[builtins.str]] = None,
                 service_name: Optional[pulumi.Input[builtins.str]] = None,
                 ssh_key: Optional[pulumi.Input[Union['InstanceSshKeyArgs', 'InstanceSshKeyArgsDict']]] = None,
                 ssh_key_create: Optional[pulumi.Input[Union['InstanceSshKeyCreateArgs', 'InstanceSshKeyCreateArgsDict']]] = None,
                 user_data: Optional[pulumi.Input[builtins.str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = InstanceArgs.__new__(InstanceArgs)

            __props__.__dict__["auto_backup"] = auto_backup
            __props__.__dict__["availability_zone"] = availability_zone
            if billing_period is None and not opts.urn:
                raise TypeError("Missing required property 'billing_period'")
            __props__.__dict__["billing_period"] = billing_period
            if boot_from is None and not opts.urn:
                raise TypeError("Missing required property 'boot_from'")
            __props__.__dict__["boot_from"] = boot_from
            __props__.__dict__["bulk"] = bulk
            if flavor is None and not opts.urn:
                raise TypeError("Missing required property 'flavor'")
            __props__.__dict__["flavor"] = flavor
            __props__.__dict__["group"] = group
            __props__.__dict__["name"] = name
            if network is None and not opts.urn:
                raise TypeError("Missing required property 'network'")
            __props__.__dict__["network"] = network
            if region is None and not opts.urn:
                raise TypeError("Missing required property 'region'")
            __props__.__dict__["region"] = region
            if service_name is None and not opts.urn:
                raise TypeError("Missing required property 'service_name'")
            __props__.__dict__["service_name"] = service_name
            __props__.__dict__["ssh_key"] = ssh_key
            __props__.__dict__["ssh_key_create"] = ssh_key_create
            __props__.__dict__["user_data"] = user_data
            __props__.__dict__["addresses"] = None
            __props__.__dict__["attached_volumes"] = None
            __props__.__dict__["flavor_id"] = None
            __props__.__dict__["flavor_name"] = None
            __props__.__dict__["image_id"] = None
            __props__.__dict__["task_state"] = None
        super(Instance, __self__).__init__(
            'ovh:CloudProject/instance:Instance',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            addresses: Optional[pulumi.Input[Sequence[pulumi.Input[Union['InstanceAddressArgs', 'InstanceAddressArgsDict']]]]] = None,
            attached_volumes: Optional[pulumi.Input[Sequence[pulumi.Input[Union['InstanceAttachedVolumeArgs', 'InstanceAttachedVolumeArgsDict']]]]] = None,
            auto_backup: Optional[pulumi.Input[Union['InstanceAutoBackupArgs', 'InstanceAutoBackupArgsDict']]] = None,
            availability_zone: Optional[pulumi.Input[builtins.str]] = None,
            billing_period: Optional[pulumi.Input[builtins.str]] = None,
            boot_from: Optional[pulumi.Input[Union['InstanceBootFromArgs', 'InstanceBootFromArgsDict']]] = None,
            bulk: Optional[pulumi.Input[builtins.int]] = None,
            flavor: Optional[pulumi.Input[Union['InstanceFlavorArgs', 'InstanceFlavorArgsDict']]] = None,
            flavor_id: Optional[pulumi.Input[builtins.str]] = None,
            flavor_name: Optional[pulumi.Input[builtins.str]] = None,
            group: Optional[pulumi.Input[Union['InstanceGroupArgs', 'InstanceGroupArgsDict']]] = None,
            image_id: Optional[pulumi.Input[builtins.str]] = None,
            name: Optional[pulumi.Input[builtins.str]] = None,
            network: Optional[pulumi.Input[Union['InstanceNetworkArgs', 'InstanceNetworkArgsDict']]] = None,
            region: Optional[pulumi.Input[builtins.str]] = None,
            service_name: Optional[pulumi.Input[builtins.str]] = None,
            ssh_key: Optional[pulumi.Input[Union['InstanceSshKeyArgs', 'InstanceSshKeyArgsDict']]] = None,
            ssh_key_create: Optional[pulumi.Input[Union['InstanceSshKeyCreateArgs', 'InstanceSshKeyCreateArgsDict']]] = None,
            task_state: Optional[pulumi.Input[builtins.str]] = None,
            user_data: Optional[pulumi.Input[builtins.str]] = None) -> 'Instance':
        """
        Get an existing Instance resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[Union['InstanceAddressArgs', 'InstanceAddressArgsDict']]]] addresses: Instance IP addresses
        :param pulumi.Input[Sequence[pulumi.Input[Union['InstanceAttachedVolumeArgs', 'InstanceAttachedVolumeArgsDict']]]] attached_volumes: Volumes attached to the instance
        :param pulumi.Input[Union['InstanceAutoBackupArgs', 'InstanceAutoBackupArgsDict']] auto_backup: Create an autobackup workflow after instance start up.
        :param pulumi.Input[builtins.str] availability_zone: The availability zone where the instance will be created
        :param pulumi.Input[builtins.str] billing_period: Billing period - hourly or monthly
        :param pulumi.Input[Union['InstanceBootFromArgs', 'InstanceBootFromArgsDict']] boot_from: Boot the instance from an image or a volume
        :param pulumi.Input[builtins.int] bulk: Create multiple instances
        :param pulumi.Input[Union['InstanceFlavorArgs', 'InstanceFlavorArgsDict']] flavor: Flavor information
        :param pulumi.Input[builtins.str] flavor_id: Flavor id
        :param pulumi.Input[builtins.str] flavor_name: Flavor name
        :param pulumi.Input[Union['InstanceGroupArgs', 'InstanceGroupArgsDict']] group: Start instance in group
        :param pulumi.Input[builtins.str] image_id: Image id
        :param pulumi.Input[builtins.str] name: Instance name
        :param pulumi.Input[Union['InstanceNetworkArgs', 'InstanceNetworkArgsDict']] network: Create network interfaces
        :param pulumi.Input[builtins.str] region: Instance region
        :param pulumi.Input[builtins.str] service_name: The id of the public cloud project. If omitted, the `OVH_CLOUD_PROJECT_SERVICE` environment variable is used
        :param pulumi.Input[Union['InstanceSshKeyArgs', 'InstanceSshKeyArgsDict']] ssh_key: Existing SSH Keypair
        :param pulumi.Input[Union['InstanceSshKeyCreateArgs', 'InstanceSshKeyCreateArgsDict']] ssh_key_create: Add existing SSH Key pair into your Public Cloud project and link it to the instance
        :param pulumi.Input[builtins.str] task_state: Instance task state
        :param pulumi.Input[builtins.str] user_data: Configuration information or scripts to use upon launch
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _InstanceState.__new__(_InstanceState)

        __props__.__dict__["addresses"] = addresses
        __props__.__dict__["attached_volumes"] = attached_volumes
        __props__.__dict__["auto_backup"] = auto_backup
        __props__.__dict__["availability_zone"] = availability_zone
        __props__.__dict__["billing_period"] = billing_period
        __props__.__dict__["boot_from"] = boot_from
        __props__.__dict__["bulk"] = bulk
        __props__.__dict__["flavor"] = flavor
        __props__.__dict__["flavor_id"] = flavor_id
        __props__.__dict__["flavor_name"] = flavor_name
        __props__.__dict__["group"] = group
        __props__.__dict__["image_id"] = image_id
        __props__.__dict__["name"] = name
        __props__.__dict__["network"] = network
        __props__.__dict__["region"] = region
        __props__.__dict__["service_name"] = service_name
        __props__.__dict__["ssh_key"] = ssh_key
        __props__.__dict__["ssh_key_create"] = ssh_key_create
        __props__.__dict__["task_state"] = task_state
        __props__.__dict__["user_data"] = user_data
        return Instance(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def addresses(self) -> pulumi.Output[Sequence['outputs.InstanceAddress']]:
        """
        Instance IP addresses
        """
        return pulumi.get(self, "addresses")

    @property
    @pulumi.getter(name="attachedVolumes")
    def attached_volumes(self) -> pulumi.Output[Sequence['outputs.InstanceAttachedVolume']]:
        """
        Volumes attached to the instance
        """
        return pulumi.get(self, "attached_volumes")

    @property
    @pulumi.getter(name="autoBackup")
    def auto_backup(self) -> pulumi.Output[Optional['outputs.InstanceAutoBackup']]:
        """
        Create an autobackup workflow after instance start up.
        """
        return pulumi.get(self, "auto_backup")

    @property
    @pulumi.getter(name="availabilityZone")
    def availability_zone(self) -> pulumi.Output[builtins.str]:
        """
        The availability zone where the instance will be created
        """
        return pulumi.get(self, "availability_zone")

    @property
    @pulumi.getter(name="billingPeriod")
    def billing_period(self) -> pulumi.Output[builtins.str]:
        """
        Billing period - hourly or monthly
        """
        return pulumi.get(self, "billing_period")

    @property
    @pulumi.getter(name="bootFrom")
    def boot_from(self) -> pulumi.Output['outputs.InstanceBootFrom']:
        """
        Boot the instance from an image or a volume
        """
        return pulumi.get(self, "boot_from")

    @property
    @pulumi.getter
    def bulk(self) -> pulumi.Output[Optional[builtins.int]]:
        """
        Create multiple instances
        """
        return pulumi.get(self, "bulk")

    @property
    @pulumi.getter
    def flavor(self) -> pulumi.Output['outputs.InstanceFlavor']:
        """
        Flavor information
        """
        return pulumi.get(self, "flavor")

    @property
    @pulumi.getter(name="flavorId")
    def flavor_id(self) -> pulumi.Output[builtins.str]:
        """
        Flavor id
        """
        return pulumi.get(self, "flavor_id")

    @property
    @pulumi.getter(name="flavorName")
    def flavor_name(self) -> pulumi.Output[builtins.str]:
        """
        Flavor name
        """
        return pulumi.get(self, "flavor_name")

    @property
    @pulumi.getter
    def group(self) -> pulumi.Output[Optional['outputs.InstanceGroup']]:
        """
        Start instance in group
        """
        return pulumi.get(self, "group")

    @property
    @pulumi.getter(name="imageId")
    def image_id(self) -> pulumi.Output[builtins.str]:
        """
        Image id
        """
        return pulumi.get(self, "image_id")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[builtins.str]:
        """
        Instance name
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def network(self) -> pulumi.Output['outputs.InstanceNetwork']:
        """
        Create network interfaces
        """
        return pulumi.get(self, "network")

    @property
    @pulumi.getter
    def region(self) -> pulumi.Output[builtins.str]:
        """
        Instance region
        """
        return pulumi.get(self, "region")

    @property
    @pulumi.getter(name="serviceName")
    def service_name(self) -> pulumi.Output[builtins.str]:
        """
        The id of the public cloud project. If omitted, the `OVH_CLOUD_PROJECT_SERVICE` environment variable is used
        """
        return pulumi.get(self, "service_name")

    @property
    @pulumi.getter(name="sshKey")
    def ssh_key(self) -> pulumi.Output[Optional['outputs.InstanceSshKey']]:
        """
        Existing SSH Keypair
        """
        return pulumi.get(self, "ssh_key")

    @property
    @pulumi.getter(name="sshKeyCreate")
    def ssh_key_create(self) -> pulumi.Output[Optional['outputs.InstanceSshKeyCreate']]:
        """
        Add existing SSH Key pair into your Public Cloud project and link it to the instance
        """
        return pulumi.get(self, "ssh_key_create")

    @property
    @pulumi.getter(name="taskState")
    def task_state(self) -> pulumi.Output[builtins.str]:
        """
        Instance task state
        """
        return pulumi.get(self, "task_state")

    @property
    @pulumi.getter(name="userData")
    def user_data(self) -> pulumi.Output[Optional[builtins.str]]:
        """
        Configuration information or scripts to use upon launch
        """
        return pulumi.get(self, "user_data")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins
import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities
from . import outputs

__all__ = [
    'GetVolumesResult',
    'AwaitableGetVolumesResult',
    'get_volumes',
    'get_volumes_output',
]

@pulumi.output_type
class GetVolumesResult:
    """
    A collection of values returned by getVolumes.
    """
    def __init__(__self__, id=None, region_name=None, service_name=None, volumes=None):
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if region_name and not isinstance(region_name, str):
            raise TypeError("Expected argument 'region_name' to be a str")
        pulumi.set(__self__, "region_name", region_name)
        if service_name and not isinstance(service_name, str):
            raise TypeError("Expected argument 'service_name' to be a str")
        pulumi.set(__self__, "service_name", service_name)
        if volumes and not isinstance(volumes, list):
            raise TypeError("Expected argument 'volumes' to be a list")
        pulumi.set(__self__, "volumes", volumes)

    @property
    @pulumi.getter
    def id(self) -> builtins.str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="regionName")
    def region_name(self) -> builtins.str:
        """
        The region name where volumes are available
        """
        return pulumi.get(self, "region_name")

    @property
    @pulumi.getter(name="serviceName")
    def service_name(self) -> builtins.str:
        """
        The id of the public cloud project.
        """
        return pulumi.get(self, "service_name")

    @property
    @pulumi.getter
    def volumes(self) -> Sequence['outputs.GetVolumesVolumeResult']:
        return pulumi.get(self, "volumes")


class AwaitableGetVolumesResult(GetVolumesResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetVolumesResult(
            id=self.id,
            region_name=self.region_name,
            service_name=self.service_name,
            volumes=self.volumes)


def get_volumes(region_name: Optional[builtins.str] = None,
                service_name: Optional[builtins.str] = None,
                opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetVolumesResult:
    """
    Get all the volume from a region of a public cloud project

    ## Example Usage

    ```python
    import pulumi
    import pulumi_ovh as ovh

    volume = ovh.CloudProject.get_volume(region_name="xxx",
        service_name="yyy")
    ```


    :param builtins.str region_name: A valid OVHcloud public cloud region name in which the volumes are available. Ex.: "GRA11".
    :param builtins.str service_name: The id of the public cloud project.
    """
    __args__ = dict()
    __args__['regionName'] = region_name
    __args__['serviceName'] = service_name
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('ovh:CloudProject/getVolumes:getVolumes', __args__, opts=opts, typ=GetVolumesResult).value

    return AwaitableGetVolumesResult(
        id=pulumi.get(__ret__, 'id'),
        region_name=pulumi.get(__ret__, 'region_name'),
        service_name=pulumi.get(__ret__, 'service_name'),
        volumes=pulumi.get(__ret__, 'volumes'))
def get_volumes_output(region_name: Optional[pulumi.Input[builtins.str]] = None,
                       service_name: Optional[pulumi.Input[builtins.str]] = None,
                       opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetVolumesResult]:
    """
    Get all the volume from a region of a public cloud project

    ## Example Usage

    ```python
    import pulumi
    import pulumi_ovh as ovh

    volume = ovh.CloudProject.get_volume(region_name="xxx",
        service_name="yyy")
    ```


    :param builtins.str region_name: A valid OVHcloud public cloud region name in which the volumes are available. Ex.: "GRA11".
    :param builtins.str service_name: The id of the public cloud project.
    """
    __args__ = dict()
    __args__['regionName'] = region_name
    __args__['serviceName'] = service_name
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('ovh:CloudProject/getVolumes:getVolumes', __args__, opts=opts, typ=GetVolumesResult)
    return __ret__.apply(lambda __response__: GetVolumesResult(
        id=pulumi.get(__response__, 'id'),
        region_name=pulumi.get(__response__, 'region_name'),
        service_name=pulumi.get(__response__, 'service_name'),
        volumes=pulumi.get(__response__, 'volumes')))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins
import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities

__all__ = [
    'GetGatewayInterfaceResult',
    'AwaitableGetGatewayInterfaceResult',
    'get_gateway_interface',
    'get_gateway_interface_output',
]

@pulumi.output_type
class GetGatewayInterfaceResult:
    """
    A collection of values returned by getGatewayInterface.
    """
    def __init__(__self__, id=None, interface_id=None, ip=None, network_id=None, region=None, service_name=None, subnet_id=None):
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if interface_id and not isinstance(interface_id, str):
            raise TypeError("Expected argument 'interface_id' to be a str")
        pulumi.set(__self__, "interface_id", interface_id)
        if ip and not isinstance(ip, str):
            raise TypeError("Expected argument 'ip' to be a str")
        pulumi.set(__self__, "ip", ip)
        if network_id and not isinstance(network_id, str):
            raise TypeError("Expected argument 'network_id' to be a str")
        pulumi.set(__self__, "network_id", network_id)
        if region and not isinstance(region, str):
            raise TypeError("Expected argument 'region' to be a str")
        pulumi.set(__self__, "region", region)
        if service_name and not isinstance(service_name, str):
            raise TypeError("Expected argument 'service_name' to be a str")
        pulumi.set(__self__, "service_name", service_name)
        if subnet_id and not isinstance(subnet_id, str):
            raise TypeError("Expected argument 'subnet_id' to be a str")
        pulumi.set(__self__, "subnet_id", subnet_id)

    @property
    @pulumi.getter
    def id(self) -> builtins.str:
        """
        ID of the gateway
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="interfaceId")
    def interface_id(self) -> builtins.str:
        """
        ID of the interface
        """
        return pulumi.get(self, "interface_id")

    @property
    @pulumi.getter
    def ip(self) -> builtins.str:
        """
        IP of the interface
        """
        return pulumi.get(self, "ip")

    @property
    @pulumi.getter(name="networkId")
    def network_id(self) -> builtins.str:
        """
        Network ID of the interface
        """
        return pulumi.get(self, "network_id")

    @property
    @pulumi.getter
    def region(self) -> builtins.str:
        """
        Region of the gateway
        """
        return pulumi.get(self, "region")

    @property
    @pulumi.getter(name="serviceName")
    def service_name(self) -> builtins.str:
        """
        ID of the cloud project
        """
        return pulumi.get(self, "service_name")

    @property
    @pulumi.getter(name="subnetId")
    def subnet_id(self) -> builtins.str:
        """
        ID of the subnet to add
        """
        return pulumi.get(self, "subnet_id")


class AwaitableGetGatewayInterfaceResult(GetGatewayInterfaceResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetGatewayInterfaceResult(
            id=self.id,
            interface_id=self.interface_id,
            ip=self.ip,
            network_id=self.network_id,
            region=self.region,
            service_name=self.service_name,
            subnet_id=self.subnet_id)


def get_gateway_interface(id: Optional[builtins.str] = None,
                          interface_id: Optional[builtins.str] = None,
                          region: Optional[builtins.str] = None,
                          service_name: Optional[builtins.str] = None,
                          opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetGatewayInterfaceResult:
    """
    Use this datasource to get a public cloud project Gateway Interface.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_ovh as ovh

    interface = ovh.CloudProject.get_gateway_interface(service_name="XXXXXX",
        region="GRA11",
        id="xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxx",
        interface_id="yyyyyyyy-yyyy-yyyy-yyyy-yyyyyyyy")
    ```


    :param builtins.str id: ID of the gateway
    :param builtins.str interface_id: ID of the interface
    :param builtins.str region: Region of the gateway
    :param builtins.str service_name: ID of the cloud project
    """
    __args__ = dict()
    __args__['id'] = id
    __args__['interfaceId'] = interface_id
    __args__['region'] = region
    __args__['serviceName'] = service_name
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('ovh:CloudProject/getGatewayInterface:getGatewayInterface', __args__, opts=opts, typ=GetGatewayInterfaceResult).value

    return AwaitableGetGatewayInterfaceResult(
        id=pulumi.get(__ret__, 'id'),
        interface_id=pulumi.get(__ret__, 'interface_id'),
        ip=pulumi.get(__ret__, 'ip'),
        network_id=pulumi.get(__ret__, 'network_id'),
        region=pulumi.get(__ret__, 'region'),
        service_name=pulumi.get(__ret__, 'service_name'),
        subnet_id=pulumi.get(__ret__, 'subnet_id'))
def get_gateway_interface_output(id: Optional[pulumi.Input[builtins.str]] = None,
                                 interface_id: Optional[pulumi.Input[builtins.str]] = None,
                                 region: Optional[pulumi.Input[builtins.str]] = None,
                                 service_name: Optional[pulumi.Input[builtins.str]] = None,
                                 opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetGatewayInterfaceResult]:
    """
    Use this datasource to get a public cloud project Gateway Interface.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_ovh as ovh

    interface = ovh.CloudProject.get_gateway_interface(service_name="XXXXXX",
        region="GRA11",
        id="xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxx",
        interface_id="yyyyyyyy-yyyy-yyyy-yyyy-yyyyyyyy")
    ```


    :param builtins.str id: ID of the gateway
    :param builtins.str interface_id: ID of the interface
    :param builtins.str region: Region of the gateway
    :param builtins.str service_name: ID of the cloud project
    """
    __args__ = dict()
    __args__['id'] = id
    __args__['interfaceId'] = interface_id
    __args__['region'] = region
    __args__['serviceName'] = service_name
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('ovh:CloudProject/getGatewayInterface:getGatewayInterface', __args__, opts=opts, typ=GetGatewayInterfaceResult)
    return __ret__.apply(lambda __response__: GetGatewayInterfaceResult(
        id=pulumi.get(__response__, 'id'),
        interface_id=pulumi.get(__response__, 'interface_id'),
        ip=pulumi.get(__response__, 'ip'),
        network_id=pulumi.get(__response__, 'network_id'),
        region=pulumi.get(__response__, 'region'),
        service_name=pulumi.get(__response__, 'service_name'),
        subnet_id=pulumi.get(__response__, 'subnet_id')))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins
import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities

__all__ = [
    'GetMitigationResult',
    'AwaitableGetMitigationResult',
    'get_mitigation',
    'get_mitigation_output',
]

@pulumi.output_type
class GetMitigationResult:
    """
    A collection of values returned by getMitigation.
    """
    def __init__(__self__, auto=None, id=None, ip=None, ip_on_mitigation=None, permanent=None, state=None):
        if auto and not isinstance(auto, bool):
            raise TypeError("Expected argument 'auto' to be a bool")
        pulumi.set(__self__, "auto", auto)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if ip and not isinstance(ip, str):
            raise TypeError("Expected argument 'ip' to be a str")
        pulumi.set(__self__, "ip", ip)
        if ip_on_mitigation and not isinstance(ip_on_mitigation, str):
            raise TypeError("Expected argument 'ip_on_mitigation' to be a str")
        pulumi.set(__self__, "ip_on_mitigation", ip_on_mitigation)
        if permanent and not isinstance(permanent, bool):
            raise TypeError("Expected argument 'permanent' to be a bool")
        pulumi.set(__self__, "permanent", permanent)
        if state and not isinstance(state, str):
            raise TypeError("Expected argument 'state' to be a str")
        pulumi.set(__self__, "state", state)

    @property
    @pulumi.getter
    def auto(self) -> builtins.bool:
        """
        Set on true if the IP is on auto-mitigation
        """
        return pulumi.get(self, "auto")

    @property
    @pulumi.getter
    def id(self) -> builtins.str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def ip(self) -> builtins.str:
        """
        The IP or the CIDR
        """
        return pulumi.get(self, "ip")

    @property
    @pulumi.getter(name="ipOnMitigation")
    def ip_on_mitigation(self) -> builtins.str:
        """
        IPv4 address
        * `permanent ` - Set on true if the IP is on permanent mitigation
        """
        return pulumi.get(self, "ip_on_mitigation")

    @property
    @pulumi.getter
    def permanent(self) -> builtins.bool:
        return pulumi.get(self, "permanent")

    @property
    @pulumi.getter
    def state(self) -> builtins.str:
        """
        Current state of the IP on mitigation
        """
        return pulumi.get(self, "state")


class AwaitableGetMitigationResult(GetMitigationResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetMitigationResult(
            auto=self.auto,
            id=self.id,
            ip=self.ip,
            ip_on_mitigation=self.ip_on_mitigation,
            permanent=self.permanent,
            state=self.state)


def get_mitigation(ip: Optional[builtins.str] = None,
                   ip_on_mitigation: Optional[builtins.str] = None,
                   opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetMitigationResult:
    """
    Use this resource to retrieve information about an IP permanent mitigation.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_ovh as ovh

    mitigation_data = ovh.Ip.get_mitigation(ip="XXXXXX",
        ip_on_mitigation="XXXXXX")
    ```


    :param builtins.str ip: The IP or the CIDR
    :param builtins.str ip_on_mitigation: IPv4 address
    """
    __args__ = dict()
    __args__['ip'] = ip
    __args__['ipOnMitigation'] = ip_on_mitigation
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('ovh:Ip/getMitigation:getMitigation', __args__, opts=opts, typ=GetMitigationResult).value

    return AwaitableGetMitigationResult(
        auto=pulumi.get(__ret__, 'auto'),
        id=pulumi.get(__ret__, 'id'),
        ip=pulumi.get(__ret__, 'ip'),
        ip_on_mitigation=pulumi.get(__ret__, 'ip_on_mitigation'),
        permanent=pulumi.get(__ret__, 'permanent'),
        state=pulumi.get(__ret__, 'state'))
def get_mitigation_output(ip: Optional[pulumi.Input[builtins.str]] = None,
                          ip_on_mitigation: Optional[pulumi.Input[builtins.str]] = None,
                          opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetMitigationResult]:
    """
    Use this resource to retrieve information about an IP permanent mitigation.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_ovh as ovh

    mitigation_data = ovh.Ip.get_mitigation(ip="XXXXXX",
        ip_on_mitigation="XXXXXX")
    ```


    :param builtins.str ip: The IP or the CIDR
    :param builtins.str ip_on_mitigation: IPv4 address
    """
    __args__ = dict()
    __args__['ip'] = ip
    __args__['ipOnMitigation'] = ip_on_mitigation
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('ovh:Ip/getMitigation:getMitigation', __args__, opts=opts, typ=GetMitigationResult)
    return __ret__.apply(lambda __response__: GetMitigationResult(
        auto=pulumi.get(__response__, 'auto'),
        id=pulumi.get(__response__, 'id'),
        ip=pulumi.get(__response__, 'ip'),
        ip_on_mitigation=pulumi.get(__response__, 'ip_on_mitigation'),
        permanent=pulumi.get(__response__, 'permanent'),
        state=pulumi.get(__response__, 'state')))
